<?php

// $Id: yubikey.inc,v 1.1.2.3 2011/02/09 05:16:44 toddejohnson Exp $

/*
 * @file
 * YubiKey utility functions.
 */

function yubikey_keyid($otp) {
  return drupal_substr($otp, 0, drupal_strlen($otp) - 32);
}

function yubikey_validate($otp, &$errstr) {
  if (strpos(variable_get('yubikey_apiurl', 'https://api.yubico.com/wsapi/verify'), '2.0') !== FALSE) {
    return yubikey_validate_2_0($otp, $errstr);
  }
  else {
    return yubikey_validate_1_1($otp, $errstr);
  }
}
function yubikey_validate_1_1($otp, &$errstr) {
  if (!variable_get('yubikey_enabled', FALSE)) {
    $errstr = "YubiKey Authentication Not Enabled";
    return FALSE;
  }

  if (!function_exists('curl_init')) {
    drupal_set_message(t('A required PHP library is missing. Check the !status_report.', array('!status_report', l('status report', 'admin/reports/status'))), 'error');
    return t('An error has occurred.');
  }

  $api_key = variable_get('yubikey_apikey', NULL);
  $id      = variable_get('yubikey_apiid', NULL);
  $timeout = variable_get('yubikey_timeout', 10);

  $api_url = variable_get('yubikey_apiurl', 'https://api.yubico.com/wsapi/verify');
  $sig_key = base64_decode($api_key);

  $hash = urlencode(base64_encode(hash_hmac("sha1", "id=$id&otp=$otp", $sig_key, TRUE)));

  $url  = "$api_url?id=$id&otp=$otp&h=$hash";

  $ch = curl_init($url);
  curl_setopt($ch, CURLOPT_TIMEOUT,        $timeout);
  curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $timeout);
  curl_setopt($ch, CURLOPT_FOLLOWLOCATION, FALSE);
  curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
  curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
  curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, TRUE);

  $result = explode("\n", curl_exec($ch));
  $error = curl_errno($ch);
  curl_close($ch);
  if ($error) {
    $errstr = "ERROR CONNECTING TO YUBICO - ". $error;
    return FALSE;
  }

  foreach ($result as $param) {
    if (drupal_substr($param, 0, 2) == "h=") {
      $signature = drupal_substr(trim($param), 2);
    }
    elseif (drupal_substr($param, 0, 2) == "t=") {
      $timestamp = drupal_substr(trim($param), 2);
    }
    elseif (drupal_substr($param, 0, 7) == "status=") {
      $status = drupal_substr(trim($param), 7);
    }
  }

  $signed = "status=". $status ."&t=". $timestamp;

  if (!(base64_encode(hash_hmac("sha1", $signed, $sig_key, TRUE)) == $signature)) {
    $errstr = 'BAD RESPONSE SIGNATURE';
    return FALSE;
  }

  if (!yubico_validate_timestamp($timestamp)) {
    $errstr = 'BAD TIMESTAMP';
    return FALSE;
  }

  if ($status != "OK") {
    $errstr = $status;
    return FALSE;
  }

  return TRUE;
}
function yubikey_validate_2_0($otp, &$errstr) {

  if (!variable_get('yubikey_enabled', FALSE)) {
    $errstr = "YubiKey Authentication Not Enabled";
    return FALSE;
  }

  $api_key = variable_get('yubikey_apikey', NULL);
  $id      = variable_get('yubikey_apiid', NULL);
  $timeout = variable_get('yubikey_timeout', 10);

  $api_url = variable_get('yubikey_apiurl', 'https://api.yubico.com/wsapi/verify');
  $sig_key = base64_decode($api_key);
  $nonce   = md5(uniqid(rand()));

  $hash = urlencode(base64_encode(hash_hmac("sha1", "id=$id&nonce=$nonce&otp=$otp", $sig_key, TRUE)));

  $url  = "$api_url?id=$id&nonce=$nonce&otp=$otp&h=$hash";

  $ch = curl_init($url);
  curl_setopt($ch, CURLOPT_TIMEOUT,        $timeout);
  curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $timeout);
  curl_setopt($ch, CURLOPT_FOLLOWLOCATION, FALSE);
  curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
  curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
  curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, TRUE);

  $result = explode("\n", curl_exec($ch));
  $error = curl_errno($ch);
  curl_close($ch);
  if ($error) {
    $errstr = "ERROR CONNECTING TO YUBICO - ". $error;
    return FALSE;
  }

  foreach ($result as $param) {
    if (drupal_substr($param, 0, 2) == "h=") {
      $signature = drupal_substr(trim($param), 2);
    }
    elseif (drupal_substr($param, 0, 6) == "nonce=") {
      $nonce = drupal_substr(trim($param), 6);
    }
    elseif (drupal_substr($param, 0, 4) == "otp=") {
      $otp = drupal_substr(trim($param), 4);
    }
    elseif (drupal_substr($param, 0, 3) == "sl=") {
      $sl = drupal_substr(trim($param), 3);
    }
    elseif (drupal_substr($param, 0, 2) == "t=") {
      $timestamp = drupal_substr(trim($param), 2);
    }
    elseif (drupal_substr($param, 0, 7) == "status=") {
      $status = drupal_substr(trim($param), 7);
    }
  }

  $signed = "nonce=$nonce&otp=$otp&sl=$sl&status=$status&t=$timestamp";
  
  if (!(base64_encode(hash_hmac("sha1", $signed, $sig_key, TRUE)) == $signature)) {
    $errstr = 'BAD RESPONSE SIGNATURE';
    return FALSE;
  }

  if (!yubico_validate_timestamp($timestamp)) {
    $errstr = 'BAD TIMESTAMP';
    return FALSE;
  }
  
  if ($status != "OK") {
    $errstr = $status;
    return FALSE;
  }

  return TRUE;
}


function yubico_validate_timestamp($timestamp) {
  $tolerance = variable_get('yubikey_tolerance', 150);
  $now = date("U");
  $timestamp_seconds = date_format(date_create(drupal_substr($timestamp, 0, -4)), "U");

  if (!$timestamp || !$now) {
    return FALSE;
  }

  if ((($timestamp_seconds + $tolerance) > $now) && ($timestamp_seconds - $tolerance) < $now) {
    return TRUE;
  }

  return FALSE;
}
